classdef EZStepper < hgsetget
    properties (SetAccess = private)
        PortObj = NaN; % COM port object; must be open
        ControllerType = 'EZHR17'; %supported: EZHR17
        ControllerName = NaN; %name or number of controller on this port
    end
    properties
        Timeout = 30; %Timeout for motor moves in sec
        LimitPolarity = 0; %0-NORMAL(ACTIVE HIGH), 1-INVERTED (ACTIVE LOW)
        LimitEnable = 1; %Enable limits
        HoldCurrent = 5; %current when not moving; in % of full (2A for EZHR17); typically 5%
        RunCurrent = 25; %current when moving; in % of full (2A for EZHR17); typically 25%
        StepDivide = 256; %number of microsteps per step; typically 256
        SlewVelocity = 100000; %Slew (max) velocity in microsteps/sec
        InitVelocity = 10000; %Initial velocity in microsteps/sec
        Ramp = 1000; %Acceleration (deceleration) in microsteps/sec^2
        Log = 0; %0 - do not accumulate log; 1 - accumulate log
        LastErr = 0; %Last error
    end
    properties (SetAccess = private, GetAccess = private) 
        nBits = 0; %Mode bits
    end
    methods
        function m = EZStepper(varargin)
            SlewVelocity = m.SlewVelocity;
            InitVelocity = m.InitVelocity;
            Ramp = m.Ramp;
            StepDivide = m.StepDivide;
            HoldCurrent = m.HoldCurrent; 
            RunCurrent = m.RunCurrent;
            LimitPolarity = m.LimitPolarity;
            LimitEnable = m.LimitEnable;
            
            property_argin = varargin;
            while (length(property_argin) >= 2)
                prop = property_argin{1};
                val = property_argin{2};
                property_argin = property_argin(3:end);
                switch prop
                case 'PortObj'
                    m.PortObj = val;
                case 'ControllerType'
                    m.ControllerType = val;
                case 'ControllerName'
                    m.ControllerName = val;
                case 'Timeout'
                    m.Timeout = val;
                case 'LimitPolarity'
                    LimitPolarity = val; 
                case 'LimitEnable'
                    LimitEnable = val; 
                case 'HoldCurrent'
                    HoldCurrent = val; 
                case 'RunCurrent'
                    RunCurrent = val; 
                case 'StepDivide'
                    StepDivide = val; 
                case 'SlewVelocity'
                    SlewVelocity = val; 
                case 'InitVelocity'
                    InitVelocity = val; 
                case 'Ramp'
                    Ramp = val; 
                case 'Log'
                    m.Log = val;
                otherwise
                    error([prop,' Is not a valid name; EZStepper properties: PortObj, ControllerType, ControllerName, Timeout, SlewVelocity, InitVelocity, Ramp, StepDivide, HoldCurrent, RunCurrent, LimitPolarity, LimitEnable, Log'])
                end
            end
            %LOAD PARAMETERS INTO CONTROLLER HERE
            %When parsing parameters must use local variables, not object properties 
            %because assignment to the properties will cause loading of parameters into controller
            %This loading can be done only later after all parameters are
            %parsed, (specifically because of log) and after the '&'
            %command is ussued and returned without error.
            [response,e0] = MotorCmndRspn(m, '&');
            if(e0==4), m.LastErr = 256; return; end
            m.SlewVelocity = SlewVelocity; e1 = m.LastErr;
            m.InitVelocity = InitVelocity; e2 = m.LastErr;
            m.Ramp = Ramp; e3 = m.LastErr;
            m.StepDivide = StepDivide; e4 = m.LastErr;
            m.HoldCurrent = HoldCurrent; e5 = m.LastErr;
            m.RunCurrent = RunCurrent; e6 = m.LastErr;
            m.LimitPolarity = LimitPolarity; e7 = m.LastErr; 
            m.LimitEnable = LimitEnable; e8 = m.LastErr;
            m.LastErr = e1 + e2*2 + e3*4 + e4*8 + e5*16 + e6*32 + e7*64 + e8*128;
        end
        function m = set.SlewVelocity(m,val)
           m.SlewVelocity = val; 
           MotorSetSlewVelo(m);
        end
        function m = set.InitVelocity(m,val)
           m.InitVelocity = val; 
           MotorSetInitVelo(m);
        end
        function m = set.Ramp(m,val)
           m.Ramp = val; 
           MotorSetRamp(m);
        end
        function m = set.StepDivide(m,val)
           m.StepDivide = val; 
           MotorSetDivide(m);
        end
        function m = set.HoldCurrent(m,val)
           m.HoldCurrent = val; 
           MotorSetHoldCurrent(m);
        end
        function m = set.RunCurrent(m,val)
           m.RunCurrent = val; 
           MotorSetRunCurrent(m);
        end
        function m = set.LimitPolarity(m,val)
           m.LimitPolarity = val; 
           MotorSetLimPolar(m);
        end
        function m = set.LimitEnable(m,val)
           m.LimitEnable = val; 
           MotorSetLimits(m);
        end
        function [response,error] = MotorCmndRspn(m, command)
            %SENDS command STRING TO EZStepper CONTROLLER AND RETURNS response STRING
            %m is an object of EZStepper class
            global EZStepperLog
            crlf=sprintf('\n');
            s = m.PortObj;
            mnum = m.ControllerName;
            cmd = ['/' int2str(mnum) command]; %FORM COMMAND
            fprintf(s,cmd); %SEND COMMAND
            res = fgets(s); %READ RESPONSE (AND CLEAR THE BUFFER)
            k = findstr('/0',res); %A VALID RESPONSE MUST CONTAIN THIS SUBSTRING
            if(isempty(k))
                response = 'Error';
                err = 4; 
            else
                ind = k(1);
                response = res(ind+2:length(res)-2); %GET RID OF EXTRA CHARACTERS 
                status = uint8(response(1:1)); %GET STATUS BYTE
                err = double(bitand(status,15)); 
                %0-No Error, 1-Initialization error, 2-Bad Command, 3-Operand out of
                %range (from previous command)
            end
            set(m,'LastErr',err);
            if(nargout > 1), error = err; end
            if(m.Log == 1), EZStepperLog = [EZStepperLog cmd res crlf]; end
            if(err ~= 0), EZStepperLog = [EZStepperLog 'Error: ' cmd res crlf]; end
        end
        function [position,error] = MotorReadPosition(m)
            %RETURNS POSITION OF EZStepper CONTROLLER.
            %m is an object of EZStepper class
            [res,err] = MotorCmndRspn(m, '?0'); %SEND COMMAND TO INTERROGATE POSITION
            spos = res(2:length(res)); %GET RID OF THE STATUS BYTE
            if(err == 0)
                position = str2double(spos);
            else
                position = NaN;
            end
            if(nargout > 1)
                if(isnan(position))
                    error = 1;
                else
                    error = 0;
                end
            end
        end
        function error = MotorMoveRel(m,travel)
            %RELATIVE MOVE FOR EZStepper CONTROLLER
            %m is an object of EZStepper class
            if (travel==0) 
                if(nargout>0), error = 0; end
                return
            end
            %ROUND TRAVEL PARAMETER
            ratrav = round(abs(travel));
            cmnd = [sprintf('%d',ratrav) 'R'];
            if (travel > 0)
                cmnd = ['P' cmnd];
            else
                cmnd = ['D' cmnd];
            end
            [res,err] = MotorCmndRspn(m,cmnd);
            if(nargout>0), error = err; end
        end
        function error = MotorMoveAbs(m,destination)
            %ABSOLUTE MOVE FOR EZHR17EN CONTROLLER.
            %m is an object of EZStepper class
            %ROUND DESTINATION PARAMETER
            rdest = round(destination);
            cmnd = ['A' sprintf('%d',rdest) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            if(nargout > 0), error = err; end
        end
        function error = MotorWait(m)
            %WAITS FOR COMPLETION OF MOTION ON EZStepper CONTROLLER.
            %m is an object of EZStepper class
            tic;
            while 1
                [moving,err] = MotorMoveStatus(m);
                if((moving==0)&&(err==0))
                    break;
                end
                if(toc > m.Timeout)
                    err = -1; break;
                end
            end
            if(nargout > 0), error = err; end
        end
        function [moving,error] = MotorMoveStatus(m)
            %RETURNS MOVING STATUS OF EZStepper CONTROLLER
            %m is an object of EZStepper class
            %moving = 0 IS NOT MOVING; moving = 1 IS MOVING.
            [res,err] = MotorCmndRspn(m, 'Q');
            status = uint8(res(1:1));
            ready = bitand(status,32);
            if(ready > 0)
                moving = 0;
            else
                moving = 1;
            end
            if(nargout > 1), error = err; end
        end
        function error = MotorJog(m,speed)
            %JOG AT GIVEN SPEED FOR EZStepper CONTROLLER.
            %m is an object of EZStepper class
            %SIGN OF speed DETERMINES DIRECTION. 
            %TO CHANGE DIRECTION CALL WITH ZERO SPEED FIRST.
            %ROUND SPEED PARAMETER
            rspd = round(speed);
            if (rspd == 0) 
                cmnd = 'T';
            end
            if (rspd > 0)
                cmnd = ['V' sprintf('%d',rspd) 'P0R'];
            end
            if (rspd < 0) 
                cmnd = ['V' sprintf('%d',-rspd) 'D0R'];
            end
            [res,err] = MotorCmndRspn(m,cmnd);
            if(nargout>0), error = err; end
        end
        function error = MotorHome(m,speed)
            %HOME COMMAND OF EZStepper CONTROLLER
            %m is an object of EZStepper class
            %START MOVING HOME AT GIVEN SPEED; DO NOT WAIT FOR COMPLETION
            %DURING HOMING MOTOR MOVES IN THE DIRECTION OF DECREASING POSITION
            spd = fix(abs(speed));
            if(spd==0)
                %error('MotorHome: zero home speed');
                if(nargout>0), error = 1; end
                return;
            end
            %SET SPEED FOR HOMING
            set(m,'SlewVelocity',spd); %MotorSetSlewVelo(m,spd);
            %DO HOMING
            [res,err] = MotorCmndRspn(m,'Z1000000000R');
            if(nargout>0), error = err; end
        end
        function error = MotorHomeComplete(m,speed)
            %HOME AND WAIT FOR COMPLETION WITH EZStepper CONTROLLER.
            %m is an object of EZStepper class
            %ORIGINAL SLEW VELOCITY IS RESTORED AFTER HOMING.
            %DURING HOMING MOTOR MOVES IN THE DIRECTION OF DECREASING POSITION.

            %READ CURRENT SLEW VELOCITY
            [slewvelo,err1] = MotorReadSlewVelo(m);
            if(err1~=0)
                if(nargout>0), error = 1; end
                return
            end
            %DON'T DO HOMING IF READING VELOCITY RETURNED ERROR
            %START HOMING
            errt = 0;
            err2 = MotorHome(m,speed);
            if(err2~=0), errt = 2; end
            %WAIT FOR COMPLETION
            err3 = MotorWait(m);
            if(err3~=0), errt = errt+4; end
            %RESTORE SLEW VELOCITY
            set(m,'SlewVelocity',slewvelo); err4=0; %err4 = MotorSetSlewVelo(m,slewvelo);
            if(err4~=0), errt = errt+8; end
            if(nargout>0), error = errt; end
        end
        function error = MotorOutputs(m,out)
            %On/Off Power Driver FOR EZStepper CONTROLLER
            %m is an object of EZStepper class
            %Interpret out as 2 bit Binary Value: 
            %3 - Both Drivers On; 2 - Driver2 On, Driver1 Off;
            %1 - Driver1 On, Driver2 Off; 0 - Both Drivers Off;
            switch out
                case {0, 1, 2, 3}
                    cmnd = ['J' sprintf('%d',out) 'R'];
                    [res,err] = MotorCmndRspn(m,cmnd);
                    if(nargout>0), error = err; end
                otherwise
                    if(nargout>0), error = -1; end
            end
        end
        function [hardwr,error] = MotorReadHardwr(m)
            %READ CURRENT STATE OF SWITCHES FOR EZStepper CONTROLLER.
            %ONE or TWO CHARACTERS AFTER STATUS ENCODE SWITCHES AND OPTOS 
            %Bit 0 = Switch 1, Bit 1 = Switch 2, Bit 2 = Opto 1, Bit 3 = Opto 2
            %m is an object of EZStepper class
            cmnd = '?4';
            [res,err] = MotorCmndRspn(m,cmnd);
            if(length(res)>1)
                n1 = str2double(res(2:2)); hardwr = n1;
            end
            if(length(res)>2)
                n2 = str2double(res(3:3)); hardwr = bitand(n1*10+n2,15);
            end
            if(nargout>1), error = err; end
        end
        function [limits,error] = MotorReadLimits(m)
            %READ CURRENT STATE OF OPTICAL LIMIT SWITCHES FOR EZStepper CONTROLLER.
            %limits=0 if no limit switches are on,
            %      =1 if limit 1 is on,
            %      =2 if limit 2 is on,
            %      =3 if both limits are on.
            %m is an object of EZStepper class
            [hardwr,err] = MotorReadHardwr(m);
            limits = fix(hardwr/4);
            if(nargout>1), error = err; end
        end         
        function [slewvelo,error] = MotorReadSlewVelo(m)
            %READ SLEW VELOCITY FOR EZStepper CONTROLLER (IN MICROSTEPS PER SEC).
            %m is an object of EZStepper class
            [res,err] = MotorCmndRspn(m,'?2');
            spos = res(2:length(res)); %GET RID OF THE STATUS BYTE
            if(err==0)
                slewvelo = str2double(spos); %IF NOT A REPRESENTATION OF A NUMBER - NaN
            else
                slewvelo = NaN;
            end
            if(nargout>1)
                if(isnan(slewvelo))
                    error = 1;
                else
                    error = 0;
                end
            end
        end
        function error = MotorSetOrigin(m,origin)
            %SET ORIGIN FOR EZStepper CONTROLLER.
            %OPTIONAL origin DEFINES SET POSITION COUNT
            %m is an object of EZStepper class
            if(nargin>1)
                org = fix(origin);
            else
                org = 0;
            end
            cmnd = ['z' sprintf('%d',org) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            if(nargout>0), error = err; end
            %SETTING ORIGIN MAY MOVE THE MOTOR TO THE NEXT FULL STEP, SO IT TAKES TIME
            pause(0.5);
        end
        function [limits,error] = MotorToLimit(m,speed)
            %MOVE UNTIL ANY LIMIT TURNS ON. RETURNS THE LIMIT THAT WAS HIT.
            %IF ALREADY AT THE LIMIT, RETURNS IMMEDIATELY
            %SIGN OF speed DETERMINES DIRECTION OF MOVE.
            %limit: 1-limit 1(A), 2-limit 2(B)
            %ORIGINAL SLEW VELOCITY IS RESTORED AFTER HOMING.
            %m is an object of EZStepper class

            %TEST LIMITS
            [limits,error] = MotorReadLimits(m);
            if((limits>0)||(error~=0))
                return;
            end
            %READ CURRENT SLEW VELOCITY
            [slewvelo,error] = MotorReadSlewVelo(m);
            if(error~=0)
                return;
            end
            %START MOVE
            error = MotorJog(m,speed);
            if(error~=0)
                MotorJog(m,0); return;
            end
            %LOOP AND CHECK
            tic;
            while(1)
                [limits,error] = MotorReadLimits(m);
                if((limits>0)||(error~=0))
                    break;
                end
                if(toc>m.Timeout)
                    error = -1; break;
                end
            end
            %STOP MOVE
            er1 = MotorJog(m,0);
            %RESTORE SLEW VELOCITY
            set(m,'SlewVelocity',slewvelo); er2=0; %er2 = MotorSetSlewVelo(m,slewvelo);
            %RETURN ERROR
            if(error~=0), return; end
            if(er1~=0), error = er1; return; end
            if(er2~=0), error = er2; return; end
        end
    end
    methods (Access = private)
        function error = MotorSetDivide(m)
            %SET MICROSTEP DIVIDE FOR EZStepper CONTROLLER
            %MICROSTEP = FULL_STEP/DIVIDE
            %VALID VALUES: 1(FULL STEP), 2, 4, 8, 16, 32, 64, 128, 256.
            %m is an object of EZStepper class
            divide = m.StepDivide;
            switch divide
                case {1, 2, 4, 8, 16, 32, 64, 128, 256}
                    %VALID VALUES
                otherwise
                    %error('Invalid microstep divide');
                    set(m,'LastErr',-1);
                    if(nargout>0), error = -1; end
                    return
            end
            cmnd = ['j' sprintf('%d',divide) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end      
        function error = MotorSetHoldCurrent(m)
            %SET HOLD CURRENT FOR EZStepper CONTROLLER (IN % OF MAX = 2 AMPS [0-50]).
            %m is an object of EZStepper class
            holdcurrent = m.HoldCurrent;
            holdcurrent = fix(holdcurrent);
            if(holdcurrent<0), holdcurrent = 0; end
            if(holdcurrent>50), holdcurrent = 50; end
            cmnd = ['h' sprintf('%d',holdcurrent) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end
        function error = MotorSetRunCurrent(m)
            %SET RUN CURRENT FOR EZStepper CONTROLLER (IN % OF MAX = 2 AMPS, [0-100]).
            %m is an object of EZStepper class
            runcurrent = m.RunCurrent;
            runcurrent = fix(runcurrent);
            if(runcurrent<0), runcurrent = 0; end
            if(runcurrent>100), runcurrent = 100; end
            cmnd = ['m' sprintf('%d',runcurrent) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end  
        end
        function error = MotorSetInitVelo(m)
            %SET INITIAL VELOCITY FOR EZStepper CONTROLLER (IN MICROSTEPS PER SEC).
            % !!! Max allowed ~15000 (in j256)
            %m is an object of EZStepper class
            initvelo = m.InitVelocity;
            ivel = fix(abs(initvelo));
            cmnd = ['v' sprintf('%d',ivel) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end
        function error = MotorSetSlewVelo(m)
            %SET SLEW VELOCITY FOR EZStepper CONTROLLER (IN MICROSTEPS PER SEC).
            %m is an object of EZStepper class
            slewvelo = m.SlewVelocity;
            svel = fix(abs(slewvelo));
            cmnd = ['V' sprintf('%d',svel) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end
        function error = MotorSetRamp(m)
            %SET ACCELERATION FOR EZStepper CONTROLLER (IN MICROSTEPS/SEC^2).
            %DECELERATION IS THE SAME AS ACCELERATION.
            %m is an object of EZStepper class
            acceleration = m.Ramp;
            accel = fix(abs(acceleration));
            cmnd = ['L' sprintf('%d',accel) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end
        function error = MotorSetLimits(m)
            %ENABLE (1) OR DISABLE (0) LIMITS FOR EZStepper CONTROLLER.
            %m is an object of EZStepper class
            %DEFINED BY BIT 1 (if counting from 0).
            enable = m.LimitEnable;
            if(enable~=0), enable=1; end
            nb = m.nBits;
            unb = uint8(nb); 
            unb = bitset(unb,2,enable);
            set(m,'nBits',unb);
            cmnd = ['n' sprintf('%d',unb) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end
        function error = MotorSetLimPolar(m)
            %SET POLARITY OF LIMIT SWITCHES FOR EZHR17EN CONTROLLER.
            %0-NORMAL(ACTIVE HIGH), 1-INVERTED (ACTIVE LOW)
            %BY DEFAULT ASSUMES THAT SWITCHES ARE NORMALLY CLOSED.
            %m is an object of EZStepper class
            limpol = m.LimitPolarity;
            cmnd = ['f' sprintf('%d',limpol) 'R'];
            [res,err] = MotorCmndRspn(m,cmnd);
            set(m,'LastErr',err);
            if(nargout>0), error = err; end
        end
    end
end