%EZStepper_Usage: PROPERTIES, METHODS, USAGE

%Before use open serial port.
[s,error] = EZStepperOpenSerial('COM1');

%After use close serial port
[error] = EZStepperCloseSerial(s);

%Create EZStepper object: 
%Two parameters, PortObj and ControllerName are required, other parameters are optional.
%PortObj is the serial port object.
%ControllerName is the controller number on this serial line.
%Values shown are typical defaults. 
%Default or supplied values are loaded into controller at object creation.
m = EZStepper('PortObj',s,'ControllerName',2,'Timeout',30,'LimitEnable',1,'StepDivide',256,'SlewVelocity',100000, ...
    'InitVelocity',10000,'Ramp',1000,'LimitPolarity',0,'HoldCurrent',5,'RunCurrent',25,'Log',0);

%Parameters and (defaults):

%Timeout - timeout for motor moves in sec (30).
%SlewVelocity - slew (max) velocity in microsteps/sec (100000).
%InitVelocity - initial velocity in microsteps/sec (10000).
%Ramp - acceleration (deceleration) in microsteps/sec^2 (1000).
%StepDivide - number of microsteps per step (256).
%HoldCurrent - hold current in percent of max (5).
%RunCurrent - run current in percent of max (25). (Max = 2A for EZHR17EN) 
%LimitPolarity - limit polarity (0). 0-switches normally open, 1-switches normally closed.
%LimitEnable - enable limits (1).
%Log - record log of every command (0);

%Get parameters - values are read from the object data structure. 
%(Ramp is used as an example)
v = get(m,'Ramp'); 
%Set parameters - values are written to the object data structure and loaded into controller.
set(m,'Ramp',v);

%Methods
[response,error] = MotorCmndRspn(m, command); %Example: MotorCmndRspn(m, 'P60000R');

[error] = MotorMoveRel(m,travel); %Relative move; travel in microsteps. 

[error] = MotorMoveAbs(m,destination); %Absolute move; destination in microsteps. 

[error] = MotorJog(m,speed); %Jog at given speed in microsteps/sec.

[error] = MotorWait(m); %Wait for completion of motion;

[moving,error] = MotorMoveStatus(m); %moving: =0 is not moving; =1 is moving.

[position,error] = MotorReadPosition(m); %returns position in microsteps.

error = MotorSetOrigin(m,origin); %Set position counter to the given value (in microsteps).
%origin is optional, if absent, assumed 0. 
%Setting origin may move motor to the next full step.

[limits,error] = MotorReadLimits(m); %limits: =0 if no limit switches are on, =1 if limit 1 is on, 
%=2 if limit 2 is on, =3 if both limits are on.

[hardwr,error] = MotorReadHardwr(m); %bit0=switch 1, bit1=switch 2, bit2=opto 1, bit3=opto 2.

[slewvelo,error] = MotorReadSlewVelo(m); %read slew velocity in microsteps/sec.

[error] = MotorOutputs(m,out); %out: 3-both power drivers on; 2-driver2 on, driver1 off; 
%1-driver1 on, driver2 off; 0-both drivers off;

[error] = MotorHome(m,speed); %Start moving home at given speed; do not wait for completion.
%During homing motor moves in the direction of decreasing position.

error = MotorHomeComplete(m,speed); %Home and wait for completion.
%Original slew velocity is restored after homing.
%During homing motor moves in the direction of decreasing position.

[limits,error] = MotorToLimit(m,speed); %Move until any limit turns on. returns the limit that was hit.
%if already at the limit, returns immediately. sign of speed determines direction of move.
%limit: 1-limit 1(a), 2-limit 2(b); original slew velocity is restored after homing.

log = EZStepperLog(); %Returns log of all executed commands while LogRecord was equal to 1.

EZStepperClearLog; %Clear accumulated log.

%GUI FUNCTIONS
EZStepper_UI - %Starts EZStepper GUI. User must use buttons to open COM port and create EZStepper object.
%All parameters are changeable by the user. 
EZStepper_UI(<parameter list>); %e.g. any or all of parameters:
EZStepper_UI('PortObj',s,'ControllerName',2,'Timeout',30,'LimitEnable',1,'StepDivide',256,'SlewVelocity',100000, ...
    'InitVelocity',10000,'Ramp',1000,'LimitPolarity',0,'HoldCurrent',5,'RunCurrent',25,'Log',0);
%Parameters that are in the list can not be changed by the user from the GUI. 
%Parameters that are not in the list can be changed by the user from the GUI.
%If Serial Port Object was created before invoking the GUI and passed to it, 
%the port does not need to (and can not) be opened. However, the user still
%must create the EZStepper object.

%THIS EXAMPLE CREATES TWO FIGURES CONTROLLING TWO EZStepper CONTROLLERS ON THE SAME SERIAL PORT
[s,err01] = EZStepperOpenSerial('COM4');
EZStepper_UI('PortObj', s, 'ControllerName', 2);
EZStepper_UI('PortObj', s, 'ControllerName', 1);

% Relevant functions:

%               EZStepperOpenSerial  EZStepperCloseSerial 
%               EZStepperLog         EZStepperClearLog  
%               EZStepper_Test*      EZStepper_UI (.m and .fig)
%               EZStepper_Usage      EZStepper
