classdef FilterWheel < hgsetget
    properties (SetAccess = private)
        PortObj; %port object; must be open
        ControllerType = 'EZHR17'; %supported: EZHR17
        ControllerName = 1; %name or number of controller on this port
        FilterPositions = 12; %number of filter positions in the wheel
        MotorSteps = 200; %physical steps in the motor; typically 200
    end
    properties
        HoldCurrent = 10; %current when not moving; in % of full (2A for EZHR17); typically 5%
        RunCurrent = 50; %current when moving; in % of full (2A for EZHR17); typically 25%
        StepDivide = 16; %number of microsteps per step; typically 16
        SlewVelocity = 10000; %slew (max) velocity in microsteps/sec 
        InitVelocity = 100; %initial velocity in microsteps/sec 
        Ramp = 12; %acceleration (deceleration) in microsteps/sec^2 
        OffsetFromHome = -2; %offset from home to the position of the first filter in microsteps
        Log = 0; %0 - do not accumulate log; 1 - accumulate log
        LastErr = 0; %Last error
    end
    properties (SetAccess = private, GetAccess = private)
        WithLimits = 0; %always without limits
        ezs; %EZStepper object
        mpf; %microsteps per filter
        ofo; %offset from origin to the first filter
        ofs; %offsets from origin to each filter
    end
    methods
        function fw = FilterWheel(varargin)
            property_argin = varargin;
            while (length(property_argin) >= 2)
                prop = property_argin{1};
                val = property_argin{2};
                property_argin = property_argin(3:end);
                switch prop
                case 'PortObj'
                    fw.PortObj = val;
                case 'ControllerType'
                    fw.ControllerType = val;
                case 'ControllerName'
                    fw.ControllerName = val;
                case 'MotorSteps'
                    fw.MotorSteps = val;
                case 'FilterPositions'
                    fw.FilterPositions = val;
                case 'OffsetFromHome'
                    fw.OffsetFromHome = val;
                case 'HoldCurrent'
                    fw.HoldCurrent = val;
                case 'RunCurrent'
                    fw.RunCurrent = val;
                case 'StepDivide'
                    fw.StepDivide = val;
                case 'SlewVelocity'
                    fw.SlewVelocity = val;
                case 'InitVelocity'
                    fw.InitVelocity = val;
                case 'Ramp'
                    fw.Ramp = val;
                case 'Log'
                    fw.Log = val;
                otherwise
                    error([prop,' Is not a valid name; FilterWheel properties: PortObj, ControllerType, ControllerName, MotorSteps, FilterPositions, OffsetFromHome, HoldCurrent, RunCurrent, StepDivide, SlewVelocity, InitVelocity, Ramp, Log'])
                end
            end
            %PARAMETER CHECKS
            %CREATE EZStepper OBJECT WITH SUPPLIED PARAMETERS
            m = EZStepper('PortObj',fw.PortObj, 'ControllerName', fw.ControllerName, ...
                'Timeout',30, 'LimitEnable',fw.WithLimits, 'StepDivide',fw.StepDivide, ...
                'SlewVelocity',fw.SlewVelocity, 'InitVelocity',fw.InitVelocity, 'Ramp',fw.Ramp, ...
                'HoldCurrent',fw.HoldCurrent, 'RunCurrent',fw.RunCurrent, 'Log',fw.Log); 
            fw.LastErr = m.LastErr;
            fw.ezs = m;
            MicrostepsPerFilter(fw);
        end % fw
        function fw = set.HoldCurrent(fw,val)
           fw.HoldCurrent = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.HoldCurrent = val; end
        end    
        function fw = set.RunCurrent(fw,val)
           fw.RunCurrent = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.RunCurrent = val; end
        end    
        function fw = set.StepDivide(fw,val)
           fw.StepDivide = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.StepDivide = val; end
        end    
        function fw = set.SlewVelocity(fw,val)
           fw.SlewVelocity = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.SlewVelocity = val; end
        end    
        function fw = set.InitVelocity(fw,val)
           fw.InitVelocity = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.InitVelocity = val; end
        end    
        function fw = set.Ramp(fw,val)
           fw.Ramp = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.Ramp = val; end
        end   
        function fw = set.Log(fw,val)
           fw.Log = val; 
           m = fw.ezs; 
           if(strcmp(class(m),'EZStepper')), m.Log = val; end
        end  
        function fw = set.OffsetFromHome(fw,val)
           fw.OffsetFromHome = val; 
        end  
        function delete(fw)
           m = fw.ezs;
           delete(m);
        end
        function err = WheelHome(fw)
            m = fw.ezs;
            err1 = MotorSetOrigin(m);
            err2 = WheelHomeOwn2(fw);
            %USE GIVEN OFFSET
            offset = fw.OffsetFromHome; %get(fw,'OffsetFromHome');
            err3 = MotorMoveRel(m,offset);
            %GET ACTUAL WORKING OFFSET FROM ORIGIN
            [pos,err4] = MotorReadPosition(m);
            fw.ofo = pos;  
            fp = fw.FilterPositions; mpf = fw.mpf;
            f = pos:mpf:pos+mpf*(fp-1);
            fw.ofs = f;
            set(fw,'SlewVelocity',get(fw,'SlewVelocity'));
            set(fw,'InitVelocity',get(fw,'InitVelocity'));
            set(fw,'Ramp',get(fw,'Ramp'));
            err = err1+err2*10+err3*100+err4*1000;
        end
        function err = WheelMove(fw, filt)
            %Move the filter wheel from the current position to the position
            %number filt. Filters are numbered from 1.
            m = fw.ezs;
            err1 = MotorWait(m);
            [pos,err2] = MotorReadPosition(m);
            apos = pos - fw.ofo;
            turnsteps = fw.MotorSteps*fw.StepDivide;
            cpos = mod(apos,turnsteps); %current position (relative to filt 1)
            tpos = (filt-1)*fw.mpf; %target position (relative to filt 1)
            if(tpos < cpos), tpos = tpos+turnsteps; end
            delta1 = tpos - cpos; delta2 = delta1 - turnsteps;
            if (abs(delta1) < abs(delta2)), delta = delta1; else delta = delta2; end
            err3 = MotorMoveRel(m,delta);
            err = err1+err2*10+err3*100;
        end
        function err = WheelNext(fw)
            %Move the filter wheel to the next filter position
            m = fw.ezs;
            err1 = MotorWait(m);
            err2 = MotorMoveRel(m,fw.mpf);
            err = err1+err2*10;
        end
        function err = WheelPrev(fw)
            %Move the filter wheel to the previous filter position
            m = fw.ezs;
            err1 = MotorWait(m);
            err2 = MotorMoveRel(m,-fw.mpf);
            err = err1+err2*10;
        end
        function err = WheelWait(fw)
            m = fw.ezs;
            err = MotorWait(m);
        end
        function [filt, err, mpos] = WheelPos(fw)
            m = fw.ezs;
            [pos,err] = MotorReadPosition(m);
            rpos = pos - fw.ofo;
            ff = rpos/fw.mpf;
            fp = round(ff);
            filt = mod(fp, fw.FilterPositions) + 1;
            if (filt <= 0), filt = filt + fw.FilterPositions; end
            if (nargout == 3), mpos = pos; end
        end
    end
    methods (Access = private)
        function MicrostepsPerFilter(fw)
            %CALCULATE MICROSTEPS PER FILTER
            fw.mpf = fw.MotorSteps*fw.StepDivide/fw.FilterPositions;    
        end
        function err = WheelHomeOwn2(fw)
           m = fw.ezs;
           command = 'v100V10000L12gD10S13G0D1gD1S03G0D1R'; %use with j16
           [resp,err1] = MotorCmndRspn(m, command);
           err2 = MotorWait(m);
           if(err1~=0), err = err1; else err = err2; end
        end
    end    
end

