classdef FocusDrive < hgsetget
    properties (SetAccess = private)
        PortObj; %port object; must be open
        ControllerType = 'EZHR17'; %supported: EZHR17
        ControllerName = 2; %name or number of controller on this port
        StrokePerRotation = 100; % up/down travel for one full turn in microns; typically 100 um.
        MotorSteps = 400; %physical steps in the motor; typically 400
        WithLimits = 0; %1 - if has limits; 0 - without limits
    end
    properties
        HoldCurrent = 5; %current when not moving; in % of full (2A for EZHR17); typically 5%
        RunCurrent = 25; %current when moving; in % of full (2A for EZHR17); typically 25%
        StepDivide = 256; %number of microsteps per step; typically 256
        SlewVelocity = 500; %slew (max) velocity in microns/sec 
        InitVelocity = 50; %initial velocity in microns/sec 
        Ramp = 0.5; %acceleration (deceleration) in microns/sec^2 
        Log = 0; %0 - do not accumulate log; 1 - accumulate log
        LastErr = 0; %Last error
    end
    properties (SetAccess = private, GetAccess = private)
        ezs; %EZStepper object
        msm; %micron to microstep conversion
    end
    methods
        function fd = FocusDrive(varargin)
            property_argin = varargin;
            while (length(property_argin) >= 2)
                prop = property_argin{1};
                val = property_argin{2};
                property_argin = property_argin(3:end);
                switch prop
                case 'PortObj'
                    fd.PortObj = val;
                case 'ControllerType'
                    fd.ControllerType = val;
                case 'ControllerName'
                    fd.ControllerName = val;
                case 'StrokePerRotation'
                    fd.StrokePerRotation = val;
                case 'MotorSteps'
                    fd.MotorSteps = val;
                case 'WithLimits'
                    fd.WithLimits = val;
                case 'HoldCurrent'
                    fd.HoldCurrent = val;
                case 'RunCurrent'
                    fd.RunCurrent = val;
                case 'StepDivide'
                    fd.StepDivide = val;
                case 'SlewVelocity'
                    fd.SlewVelocity = val;
                case 'InitVelocity'
                    fd.InitVelocity = val;
                case 'Ramp'
                    fd.Ramp = val;
                case 'Log'
                    fd.Log = val;
                otherwise
                    error([prop,' Is not a valid name; FocusDrive properties: PortObj, ControllerType, ControllerName, StrokePerRotation, MotorSteps, WithLimits, HoldCurrent, RunCurrent, StepDivide, SlewVelocity, InitVelocity, Ramp, Log'])
                end
            end
            %PARAMETER CHECKS
            % RECALCULATE FROM MICRONS TO MICROSTEPS
            MicronToMicrostep(fd);           
            slewmsm = fix(fd.SlewVelocity*fd.msm);
            initmsm = fix(fd.InitVelocity*fd.msm);
            rampmsm = fix(fd.Ramp*fd.msm);
            %CREATE EZStepper OBJECT WITH SUPPLIED PARAMETERS
            m = EZStepper('PortObj',fd.PortObj, 'ControllerName', fd.ControllerName, ...
                'Timeout',30, 'LimitEnable',fd.WithLimits, 'StepDivide',fd.StepDivide, ...
                'SlewVelocity',slewmsm, 'InitVelocity',initmsm, 'Ramp',rampmsm, ...
                'HoldCurrent',fd.HoldCurrent, 'RunCurrent',fd.RunCurrent, 'Log',fd.Log); 
            fd.LastErr = m.LastErr;
            fd.ezs = m;
        end % fd
        function fd = set.HoldCurrent(fd,val)
           fd.HoldCurrent = val; 
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.HoldCurrent = val; end
        end    
        function fd = set.RunCurrent(fd,val)
           fd.RunCurrent = val; 
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.RunCurrent = val; end
        end    
        function fd = set.StepDivide(fd,val)
           fd.StepDivide = val; 
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.StepDivide = val; end
           MicronToMicrostep(fd);
        end    
        function fd = set.SlewVelocity(fd,val)
           fd.SlewVelocity = val; 
           slewmsm = fix(fd.SlewVelocity*fd.msm);
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.SlewVelocity = slewmsm; end
        end    
        function fd = set.InitVelocity(fd,val)
           fd.InitVelocity = val; 
           initmsm = fix(fd.InitVelocity*fd.msm);
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.InitVelocity = initmsm; end
        end    
        function fd = set.Ramp(fd,val)
           fd.Ramp = val; 
           rampmsm = fix(fd.Ramp*fd.msm);
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.Ramp = rampmsm; end
        end   
        function fd = set.Log(fd,val)
           fd.Log = val; 
           m = fd.ezs; 
           if(strcmp(class(m),'EZStepper')), m.Log = val; end
        end  
        function delete(fd)
           m = fd.ezs;
           delete(m);
        end
        function err = ZRelative(fd,travel)
            travsteps = fix(travel*fd.msm);
            m = fd.ezs;
            err = MotorMoveRel(m,travsteps);
        end
        function err = ZAbsolute(fd,destination)
            deststeps = fix(destination*fd.msm);
            m = fd.ezs;
            err = MotorMoveAbs(m,deststeps);
        end
        function err = ZJog(fd, speed)
            stepspeed = fix(speed*fd.msm);
            m = fd.ezs;
            err = MotorJog(m,stepspeed);
        end
        function err = ZStop(fd)
            m = fd.ezs;
            err = MotorJog(m,0);
        end
        function err = ZHome(fd, homespeed)
            if (fd.WithLimits == 1)
                hstepspeed = fix(homespeed*fd.msm);
                m = fd.ezs;
                err  = MotorHome(m,hstepspeed);
            else
                err = 1;
            end
        end
        function err = ZHomeComplete(fd, homespeed)
            if (fd.WithLimits == 1)
                hstepspeed = fix(homespeed*fd.msm);
                m = fd.ezs;
                err  = MotorHomeComplete(m,hstepspeed);
            else
                err = 1;
            end
        end
        function err = ZWait(fd)
            m = fd.ezs;
            err = MotorWait(m);
        end
        function err = ZOrigin(fd, origin)
            if(nargin > 1)
                orig = origin;
            else
                orig = 0;
            end
            m = fd.ezs;
            origms = fix(orig*fd.msm);
            err = MotorSetOrigin(m,origms);
        end
        function [pos, err] = ZPos(fd)
            m = fd.ezs;
            [position,err] = MotorReadPosition(m);
            pos = position/fd.msm;
        end
    end
    methods (Access = private)
        function MicronToMicrostep(fd)
            %MICRON TO MICROSTEP CONVERSION: msm - number of microsteps in 1 micron
            fd.msm = fd.MotorSteps*fd.StepDivide/fd.StrokePerRotation;    
        end
    
    end    
end

