%FocusDrive_Usage: PROPERTIES, METHODS, USAGE

%Before use open serial port.
[s,error] = EZStepperOpenSerial('COM1');

%After use close serial port
[error] = EZStepperCloseSerial(s);

%Create FocusDrive object: 
%Two parameters, PortObj and ControllerName are required, other parameters are optional.
%PortObj is the serial port object.
%ControllerName is the controller number on this serial line.
%Values shown are typical defaults. 
%Default or supplied values are loaded into controller at object creation.
fd=FocusDrive('PortObj', s, 'ControllerName', 2, 'StrokePerRotation', 100, 'MotorSteps', 400, 'WithLimits', 0, ...
    'HoldCurrent', 5, 'RunCurrent', 25, 'StepDivide', 256, 'SlewVelocity', 500, 'InitVelocity', 10, 'Ramp', 0.75, 'Log', 0);

%Parameters and (defaults):

StrokePerRotation = 100; % up/down travel for one full turn in microns; typically 100 um.
MotorSteps = 400; %physical steps in the motor; typically 400
WithLimits = 0; %1 - if has limits; 0 - without limits
HoldCurrent = 5; %current when not moving; in % of full (2A for EZHR17); typically 5%
RunCurrent = 25; %current when moving; in % of full (2A for EZHR17); typically 25%
StepDivide = 256; %number of microsteps per step; typically 256
SlewVelocity = 500; %slew (max) velocity in microns/sec 
InitVelocity = 50; %initial velocity in microns/sec 
Ramp = 0.5; %acceleration (deceleration) in microns/sec^2 
Log = 0; %0 - do not accumulate log; 1 - accumulate log

%Get parameters - values are read from the object data structure. 
%(Ramp is used as an example)
v = get(fd,'Ramp'); 
%Set parameters - values are written to the object data structure and loaded into controller.
set(fd,'Ramp',v);

%Methods
err = ZRelative(fd,travel); %Relative move; travel in microns.

err = ZAbsolute(fd,destination); %Absolute move; destination in microns.

err = ZJog(fd, speed); %Jog at given speed in microns/sec.

err = ZStop(fd); %Stop movement (same as Jog with speed=0)

err = ZHome(fd, homespeed); %Start moving home at given speed; do not wait for completion.
%During homing motor moves in the direction of decreasing position.

err = ZHomeComplete(fd, homespeed); %Home and wait for completion.
%Original slew velocity is restored after homing.
%During homing motor moves in the direction of decreasing position.

err = ZWait(fd); %Wait for completion of motion;

err = ZOrigin(fd, origin); %Set position to the given value (in microns), if absent, =0. 

[pos, err] = ZPos(fd); %Returns current position in microns.

log = EZStepperLog(); %Returns log of all executed commands while LogRecord was equal to 1.

EZStepperClearLog; %Clear accumulated log.

%GUI FUNCTIONS
FocusDrive_UI - %Starts FocusDrive GUI. User must use menus to open COM port and create FocusDrive object.
%All parameters are changeable by the user. At start GUI figure reads parameter file FocusDrive.mat from the current 
%directory (if present). Parameter file is (re)written after object creation.
FocusDrive_UI(<parameter list>); %e.g. any or all of parameters:
FocusDrive_UI('PortObj', s, 'ControllerName', 2, 'StrokePerRotation', 100, 'MotorSteps', 400, 'WithLimits', 0, ...
    'HoldCurrent', 5, 'RunCurrent', 25, 'StepDivide', 256, 'SlewVelocity', 500, 'InitVelocity', 10, 'Ramp', 0.75, 'Log', 0);
%Parameters that are in the list can not be changed by the user from the GUI. 
%Parameters that are not in the list can be changed by the user from the GUI.
%Parameters from the command line override parameters read from the file.
%If Serial Port Object was created before invoking the GUI and passed to
%it, the port does not need to (and can not) be opened. However, the user still
%must create the FocusDrive object in the menu: Initialize/Create FD Object.

%THIS EXAMPLE CREATES TWO FIGURES CONTROLLING TWO FOCUS DRIVES ON THE SAME SERIAL PORT
[s,err01] = EZStepperOpenSerial('COM1');
FocusDrive_UI('PortObj', s, 'ControllerName', 2);
FocusDrive_UI('PortObj', s, 'ControllerName', 1);

% Relevant functions:

%               EZStepperOpenSerial  EZStepperCloseSerial 
%               EZStepperLog         EZStepperClearLog  
%               FocusDrive_Test*     FocusDrive_UI (.m and .fig)
%               FocusDrive_Usage     FocusDrive
